from infinishell.infishell.commands import commandgroup, command, internal, render
from infinishell.infishell.rlcontext import get_active_rlcontext
from infinishell.infishell.utils import strlist

from infinishell.infishell import (
    config, datadir,
    )

from ...environment import variables as env
from ...utils import get_username_password
from ..system import system
from ..remote import clear_remote_system_cache

from .options import Grep, InvertedGrep
from .types import UserStr

from munch import Munch

@commandgroup(prefix="")
class GeneralCommands(object):
    @internal
    @command(
        description="Login as another user",
        types=(UserStr,),
        )
    def login(self, _api, username=None):
        username, password = get_username_password(username)
        if isinstance(username, dict):
            username = username.name
        system.login(username, password,
                     logout_previous_user=not env.is_set('AUTH_JSESSIONID'))
        clear_remote_system_cache()
        rlcontext = get_active_rlcontext(safe=True)
        if rlcontext is not None:
            rlcontext.clear_completion_cache()
            history_filename = self._get_history_filename(system.name(), system.user.username)
            rlcontext.load_history_for_user(history_filename=history_filename)


    def _get_history_filename(self, address, user):
        if not config.history:
            return None
        if config.history_file:
            return config.history_file
        address = address.lower()
        simplified_address = "".join(c for c in address if c in ("abcdefghijklmnopqrstuvwxyz1234567890-_"))
        simplified_address = "{}_{}_{:x}".format(simplified_address, user, abs(hash(address)))

        return datadir.get_path("history", simplified_address)

    @internal
    @command(
        description="Show user history",
        options=(Grep(), InvertedGrep(),),
        types=(),
        )
    @render(
        fields=strlist("num command"),
        substitute=dict(num='---'),
        )

    def history(self, _api):
        rlcontext = get_active_rlcontext(safe=True)
        if rlcontext is not None:
            return [Munch(num=ind+1,command=unicode(elem, 'utf-8') if isinstance(elem, str) else elem)
                    for ind, elem in enumerate(rlcontext._get_history())]
        else:
            return []