from munch import Munch, unmunchify
import time


DEFAULT_DICT = Munch()

_runtimeConfig = None


class RuntimeConfig(object):

    @staticmethod
    def get_instance():
        global _runtimeConfig
        if _runtimeConfig is None:
            _runtimeConfig = RuntimeConfig()
            _runtimeConfig._init_dict()
        return _runtimeConfig

    def __init__(self):
        self._config_dict = Munch(mgmt_config={})
        self._system = None
        self._time = None

    def _init_dict(self):
        self._set_mgmt_config()
        self._config_dict.update(DEFAULT_DICT)

    def _throttle(func):

        def wrapper(self):
            if self._system is None:
                return
            now = time.time()
            if self._time is None:
                self._time = now
                func(self)
            else:
                if now - self._time > 5:
                    self._time = now
                    func(self)
                else:
                    pass
        return wrapper

    @_throttle
    def _set_mgmt_config(self):
        try:
            config = self._system.api.get('config')
            self._config_dict.mgmt_config = config
        except AttributeError as e:
            self._config_dict.mgmt_config = {}

    def set_system(self, system, clear_config=True):
        self._system = system
        if clear_config:
            self.reset()

    def reset(self):
        self._init_dict()

    def reset_mgmt(self):
        self._set_mgmt_config()

    def __getattr__(self, attr):
        self._set_mgmt_config()
        return getattr(self._config_dict, attr)

    def __setattr__(self, attr, value):
        if attr in ('_config_dict', '_system'):
            super(RuntimeConfig, self).__setattr__(attr, value)
        elif attr == '_time':
            super(RuntimeConfig, self).__setattr__(attr, value)
        else:
            assert attr in ['mgmt_config'], ('Cannot set %s attribute '
                                             'manually' % attr)
            setattr(self._config_dict, attr, value)

    def __getitem__(self, key):
        self._set_mgmt_config()
        return self._config_dict.get(key,{})

    def __setitem__(self, key, value):
        assert key in ['mgmt_config'], ('Cannot set %s attribute '
                                        'manually' % key)
        self._config_dict[key] = value

    def __contains__(self, key):
        return key in self._config_dict

    def __iter__(self):
        return iter(self._config_dict)

    def __repr__(self):
        return str(unmunchify(self._config_dict))

