from __future__ import absolute_import

from itertools import chain, izip
import csv
from numbers import Number

from infinishell.infishell.shell import get_active_shell

from .renderer import RendererBase
from .values import ValueRenderer

class COUNTRenderer(RendererBase):
    DEFAULT_SEPARATOR = ","
    def __init__(self, *args, **kwargs):
        super(COUNTRenderer, self).__init__(*args, **kwargs)
        self._value_renderer = ValueRenderer(empty_value="")
    def _render_impl(self, obj):
        if obj is None:
            self._writeln(str(0))
            return
        if isinstance(obj, (basestring, Number)):
            self._writeln(str(obj))
            return
        elif self._properties.get("individually", False) and isinstance(obj, dict):
            obj = [obj]
        self._render_collection(obj)
    def _render_collection(self, objects):
        it = iter(objects)
        try:
            first_object = next(it)
        except StopIteration:
            self._writeln(str(0))
            return
        all_objects = chain([first_object], it)
        amount = 0
        for obj in all_objects:
            amount += 1
        print "%s" % amount

def toggle_count_mode(shell=None):
    shell = shell or get_active_shell()
    if not is_count_mode(shell):
        set_count_mode(True, shell)
        return True
    set_count_mode(False, shell)
    return False

def is_count_mode(shell=None):
    return isinstance((shell or get_active_shell()).get_renderer_override(), COUNTRenderer)

def set_count_mode(enabled, shell=None):
    shell = shell or get_active_shell()
    if enabled:
        shell.set_renderer_override(COUNTRenderer())
        shell.disable_stream_encoding()
    else:
        shell.set_renderer_override(None)
        shell.enable_stream_encoding()
