
from infinishell.infishell import commandgroup, command
from infinishell.infishell.utils import strlist
from . import render_collection
from .types import Bbu
from .options import QueryOptions
from . import EntityCommands, roles


QUERY_FIELDS = strlist("name onBattery charging battery_charge")
EXTENDED_QUERY_FIELDS = QUERY_FIELDS + strlist("load_level_percent load_level_watt output_current")
SUBSTITUTE = dict(
    onBattery='on_batt',
    load_level_percent='load(%)',
    load_level_watt='load(w)',
    battery_charge='charge_lvl(%)',
    output_current='output_current(A)'
)

@commandgroup(prefix="bbu")
class BbuCommands(object):
    pass


@commandgroup(prefix="health", parent=BbuCommands)
class BbuHealthCommands(object):
    pass

@commandgroup(prefix="power", parent=BbuHealthCommands)
class BbuPowerCommands(EntityCommands):
    ENTITY_TYPE = Bbu

    @command(
        name="query",
        description="List system BBUs",
        options=QueryOptions(fields=EXTENDED_QUERY_FIELDS, substitute=SUBSTITUTE),
        substitute=SUBSTITUTE,
    )
    @render_collection(
        entity=Bbu,
        fields=EXTENDED_QUERY_FIELDS,
        substitute=SUBSTITUTE,
    )
    @roles.INFINIDAT.TECHNICIAN
    def extended_query(self, _api):
        return self._query(_api, _hook=self._query_hook)
    
    @command(
        description="List system BBUs",
        options=QueryOptions(fields=QUERY_FIELDS, substitute=SUBSTITUTE),
        substitute=SUBSTITUTE,
    )
    @render_collection(
        entity=Bbu,
        fields=QUERY_FIELDS,
        substitute=SUBSTITUTE,
    )
    @roles.ADMIN.POOLADMIN.READONLY
    def query(self, _api):
        return self._query(_api, _hook=self._query_hook)
    
    def _query_hook(self, bbus):
        for bbu in bbus:
            bbu.name= 'bbu-' + str(bbu.id)
            yield bbu
