from .commands import commandgroup, command, magic
from ..exceptions import CommandFailed, ShellException
from ..types import Str, Bool, MappedEnumType
from ..shell import get_active_shell
from ..config import config
from ..console import get_console_size

class LoggingMode(MappedEnumType):
    def _get_mapping(self):
        return dict(append="a", overwrite="w")

@commandgroup(manifest="__internal__")
class MagicCommands(object):
    def __init__(self, manager):
        super(MagicCommands, self).__init__()
        self._manager = manager
    @magic
    @command(types=(Str, LoggingMode, Bool, Bool),
             description="Start command logger")
    def logstart(self, _api, file=None, mode=None, output=False, timestamps=False):
        if file is None:
            file = config.log_file
            if file is None:
                raise CommandFailed("Please specify log file, as no default was set")
        shell = get_active_shell()
        if shell.is_logging():
            shell.error("Already logging to {filename}".format(**shell.get_logging_properties()))
            return
        try:
            shell.start_logging(filename=file, mode=mode, with_output=output, with_timestamps=timestamps)
        except ShellException as e:
            raise CommandFailed(e)
        return "Logger activated"
    @magic
    @command(description="Stop command logger")
    def logstop(self, _api):
        shell = get_active_shell()
        if not shell.is_logging():
            shell.error("Logger is already inactive")
            return
        shell.stop_logging()
        return "Logger deactivated"
    @magic
    @command(description="Display command logger state")
    def logstate(self, _api):
        props = get_active_shell().get_logging_properties()
        return "Logging is off" if not props else "Logging to {}".format(props.filename)
    @magic
    @command(description="Clear screen")
    def clear(self, _api):
        return "\n" * (get_console_size()[1] - 2)
