class ShellException(Exception):
    """ Base class for all shell exceptions. """
    pass

class FatalShellError(ShellException):
    pass

class BadCommandDefinition(ShellException):
    pass

class BadSyntax(ShellException):
    def __init__(self, msg, line="", lineno=None, loc=None, suggestions=None):
        self.msg = msg
        self.line = line
        self.lineno = lineno
        self.loc = loc
        self.column = None if loc is None else loc + 1
        self.suggestions = suggestions or []
        if (self.lineno, self.loc).count(None) == 2:
            full_msg = self.msg
        elif self.lineno is None:
            full_msg = "{} (character {})".format(self.msg, self.column)
        elif self.loc is None:
            full_msg = "{} (line {})".format(self.msg, self.lineno)
        else:
            full_msg = "{} (line {}, character {})".format(self.msg, self.lineno, self.column)
        if suggestions:
            full_msg = "{}. Did you mean: {}?".format(full_msg, ", ".join(suggestions))
        super(BadSyntax, self).__init__(full_msg)

class NoSuchCommand(BadSyntax):
    pass

class BadArguments(BadSyntax):
    pass

class BadValue(BadArguments):
    pass

class UnsupportedCharacterSet(BadValue):
    pass

class CommandFailed(ShellException):
    """ Command-related exception. Handling of this exception and its subclasses
    is always contained within the shell logic, unlike other
    :py:class:`.ShellException` subclasses, which may cause the shell to
    terminate, or otherwise stop the default shell flow. """
    pass

class CommandAborted(CommandFailed):
    pass

class OutputError(ShellException):
    pass

class NoActiveReadlineContext(Exception):
    pass

class TooManyPossibleValues(ShellException):
    pass

class FilesystemException(ShellException):
    pass

class SpecialCaseException(ShellException):
    def __init__(self, **kwargs):
        self.__dict__.update(kwargs)
