from contextlib import contextmanager

from .nothing import NOTHING

class ContextProxy(object):

    __allowed_attrs__ = set(('__contexts__', '__proxy_name__', '__exception_class__'))

    def __init__(self, name='context', exception_class=AssertionError):
        self.__proxy_name__ = name
        self.__exception_class__ = exception_class
        self.__contexts__ = []

    def __get_context__(self, safe=False):
        if not self.__contexts__:
            return None if safe else self.__raise_exception__()
        return self.__contexts__[-1]

    def __push_context__(self, ctx):
        self.__contexts__.append(ctx)

    def __pop_context__(self, ctx=NOTHING):
        if not self.__contexts__:
            self.__raise_exception__()
        popped = self.__contexts__.pop()
        if ctx is not NOTHING:
            assert popped is ctx, 'Popped context is unexpected:\nExpected: {}\nActual: {}'\
                                  .format(ctx, popped)

    def __is_context_active__(self):
        return bool(self.__contexts__)

    def __raise_exception__(self):
        raise self.__exception_class__('No active ' + self.__proxy_name__)

    @contextmanager
    def __as__(self, ctx):
        self.__push_context__(ctx)
        try:
            yield
        finally:
            self.__pop_context__(ctx)

    def __getattr__(self, attr):
        return getattr(self.__get_context__(), attr)

    def __setattr__(self, attr, value):
        if attr in self.__allowed_attrs__:
            return object.__setattr__(self, attr, value)
        setattr(self.__get_context__(), attr, value)

    def __getitem__(self, item):
        return self.__get_context__()[item]

    def __setitem__(self, item, value):
        self.__get_context__()[item] = value
