import sys
import os
import atexit
__all__ = ["getch"]

class _Getch:
    def __init__(self):
        self.impl = None
        for impl_class in (_GetchWindows, _GetchUnix):
            try:
                self.impl = impl_class()
            except:
                pass
    def __call__(self):
        returned = self.impl()
        if isinstance(returned, bytes):
            returned = returned.decode('utf-8')
        return returned



class _GetchUnix:
    def __init__(self):
        import tty, termios
        if sys.stdin.isatty():
            self._tty_fd = sys.stdin.fileno()
        else:
            tty_filename = os.ctermid()
            self._tty_fd = os.open(tty_filename, os.O_RDONLY)
            atexit.register(self._atexit)
    def _atexit(self):
        try: os.close(self._tty_fd)
        except: pass
    def __call__(self):
        import tty, termios
        old_settings = termios.tcgetattr(self._tty_fd)
        try:
            tty.setraw(self._tty_fd)
            ch = os.read(self._tty_fd, 1)
        finally:
            termios.tcsetattr(self._tty_fd, termios.TCSADRAIN, old_settings)
        return ch

class _GetchWindows:
    def __init__(self):
        import msvcrt
    def __call__(self):
        import msvcrt
        return msvcrt.getch()

getch = _Getch()

is_available = getch.impl is not None
