from .getch import getch, is_available as is_getch_available
from ..console import get_console_size
from itertools import count
from munch import Munch
from colorama import Fore, Style

# TODO: This module should either be deprecated, or merged with infinishell.infinibox.streams.pagination

is_available = is_getch_available

CHARS = Munch(
    print_all = "g",
    quit = "q",
    )
_KEYS = ["Space", "Enter"] + [c.upper() for c in CHARS.itervalues()]
_PROMPT = "More..."
MORE_RESULTS_PROMPT = "{} ({})".format(_PROMPT, ", ".join(_KEYS))
_COLORED_KEYS = [Fore.BLUE + key + Fore.RESET for key in _KEYS]
COLORED_MORE_RESULTS_PROMPT = Style.BRIGHT + "{} ({})".format(_PROMPT, ", ".join(_COLORED_KEYS)) + Style.RESET_ALL
_prompt_len = len(MORE_RESULTS_PROMPT)
ERASE_PROMPT_STR = "\10"*_prompt_len + " "*_prompt_len + "\10"*_prompt_len

_enum = count()
ACTIONS = Munch(
    page_down = next(_enum),
    down = next(_enum),
    quit = next(_enum),
    print_all = next(_enum),
    )

class More(object):
    def more(self, lines, stream, color=False):
        console_width, console_height = get_console_size()
        console_height -= 1 # Leave one line for the prompt
        page_weight = 0
        print_all = False
        try:
            for line in lines:
                if print_all:
                    stream.write(line)
                    stream.write("\n")
                    continue
                line_length = len(line) - 1 if line.endswith("\n") else len(line)
                line_weight = (1 if not line_length
                               else line_length / console_width + 1 if line_length % console_width
                               else line_length / console_width)
                page_weight += line_weight
                if page_weight > console_height:
                    action = self._get_action(stream, color)
                    if action is ACTIONS.page_down:
                        page_weight = line_weight
                    elif action is ACTIONS.print_all:
                        print_all = True
                    elif action is ACTIONS.quit:
                        return
                    elif action is ACTIONS.down:
                        pass
                stream.write(line)
                stream.write("\n")
        finally:
            stream.flush()
    def _get_action(self, stream, color):
        stream.write(COLORED_MORE_RESULTS_PROMPT if color else MORE_RESULTS_PROMPT)
        stream.flush()
        while True:
            c = getch().lower()
            if c == " ":
                action = ACTIONS.page_down
                break
            if c in ("\r", "\n"):
                action = ACTIONS.down
                break
            if c == CHARS.print_all:
                action = ACTIONS.print_all
                break
            if c == CHARS.quit:
                action = ACTIONS.quit
                break
        stream.write(ERASE_PROMPT_STR)
        return action

_more = More()

def more(*args, **kwargs):
    return _more.more(*args, **kwargs)
