import errno

__all__ = ['UnicodeSafeStreamWrapper']

class UnicodeSafeStreamWrapper(object):
    ''' Stream wrapper that encodes output in UTF-8. '''
    def __init__(self, stream):
        self.__stream = stream
        encoding = getattr(stream, "encoding", None) or "UTF-8"
        self.__prewrite = lambda s: s if isinstance(s, str) else s.encode(encoding, "ignore")
        self._enabled = True
    def __getattr__(self, attr):
        return getattr(self.__stream, attr)
    def write(self, b):
        if not self.__stream.closed:
            if self._enabled:
                b = self.__prewrite(b)
            try:
                self.__stream.write(b)
            except IOError as e:
                if e.errno == errno.EPIPE:
                    self.__stream.close()
    @property
    def closed(self):
        return self.__stream.closed
    def close(self):
        self.__stream.close()
    def enable_encoding(self):
        self._enabled = True
    def disable_encoding(self):
        self._enabled = False
    def is_encoding_enabled(self):
        return self._enabled
    def __repr__(self):
        return "<{}: {!r}>".format(type(self).__name__, self.__stream)
